@@ -2,19 +2,21 @@
 ## Generalized CWM ##
 #####################
 
-cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,colXp,Xbtrials, n, m, Xmod, k, modelXnorm, familyY,familyYname,mY, 
-                    method, initialization, start.z, iter.max, threshold, loglikplot, seed,maxR) 
+cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,colXp,Xbtrials, n, m, Xmod, k, modelXnorm, familyY, 
+                    method, initialization, start.z, iter.max, threshold, loglikplot, seed,maxR,eps) 
   # colXm number of factors in data
   # colXn number of Xnorm variables in data
   # m vector with number of levels of factors
 {
   t_df <- vY <- NULL       # only for the t distribution  
   nuY <- NULL       # only for the Gamma distribution   
-  
+  familyYname <- familyY$family  
   if(!is.null(seed)) set.seed(seed) 
   if(initialization=="manual" |initialization=="mclust") maxR <- 1
   
   
+  
+  
   # Xnorm Parameters definition ---------------------------------------------------------
   if(colXn>0){
   }  
@@ -22,9 +24,13 @@ cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,
   prior   <- numeric(k) # weights
   
   
-  if(!is.null(data)) if(familyYname=="t"){ 
+  if(!is.null(data)){ 
+    if(familyYname=="student.t"){ 
     vY    <- array(1,c(n,k),dimnames=list(1:n,paste("comp.",1:k,sep="")))
     t_df     <- rep(30,k)   # only for the t distribution
+    }
+    if (familyYname=="binomial" && !is.factor(Y)) mY <- rowSums(Y)
+    else mY <- rep(1,nrow(Y))
   }
   result <- list()
   # Do Estimation ----------------------------------------------------------------
@@ -48,7 +54,7 @@ cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,
       nj    <- colSums(z) #for multinomial variables
       
       # Posterior probabilities
-      norm  <- .PX_norm(colXn=colXn,Xnorm=Xnorm,modelXnorm=modelXnorm,z=z,k=k,n=n)
+      norm  <- .PX_norm(colXn=colXn,Xnorm=Xnorm,modelXnorm=modelXnorm,z=z,k=k,n=n,eps=eps)
       bin   <- .PX_bin(k=k,X=Xbin,weights=z,Xbtrials=Xbtrials,n=n)
       pois  <- .PX_Poisson(k=k,X=Xpois,weights=z,n=n) 
       multi <- .PX_multi(colXm=colXm,Xmod=Xmod,z=z,nj=nj,m=m,k=k,n=n)
@@ -61,11 +67,11 @@ cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,
       # Global - Observed-data log-likelihood ## 
       dens              <- matrix(rep(prior,n),n,k,byrow=TRUE)*norm$PX*multi$PX*pois$PX*bin$PX*l$PY
       loglik[iteration] <- sum(log(rowSums(dens)))
-      
+        
       # Aitkane's Acceleration-Based Stopping Criterion # #
       
       if(iteration>2 & k > 1){
-        if (!is.na(loglik[iteration-1]-loglik[iteration-2])){
+        if (!is.na(loglik[iteration-1]-loglik[iteration-2]) && !is.na(loglik[iteration])){
           if(abs(loglik[iteration-1]-loglik[iteration-2])>0){
             a[iteration-1]      <- (loglik[iteration]-loglik[iteration-1])/(loglik[iteration-1]-loglik[iteration-2])
             aloglik[iteration]  <- loglik[iteration-1]+(1/(1-a[iteration-1])*(loglik[iteration]-loglik[iteration-1]))
@@ -85,7 +91,7 @@ cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,
       dens[dens==0] <- 1E-320
       z      <- dens/matrix(rep(rowSums(dens) ,k),ncol=k)                     # (n x k)
       
-      if(!is.null(data)) if(familyYname=="t"){
+      if(!is.null(data)) if(familyYname=="student.t"){
         t_df <- l$t_df
         normzvY <- l$zvY/matrix(rep(colSums(l$zvY),n),n,k,byrow=TRUE) 
         for(j in 1:k)
@@ -103,9 +109,10 @@ cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,
     df_model <- length(coef(l$lmodelY[[1]]))*k
     if(!is.null(data)){
       if(familyYname=="gaussian" | familyYname=="Gamma")  df_model <- df_model + k
-      if(familyYname=="t")  df_model <- df_model + 2*k
-      if (!is.null(modelXnorm)) df_model <- df_model + ncovpar(modelname=strtrim(paste0(modelXnorm,"II"), 3), p=colXn, G=k)    
+      if(familyYname=="student.t")  df_model <- df_model + 2*k
     }
+    if (!is.null(modelXnorm)) df_model <- df_model + ncovpar(modelname=strtrim(paste0(modelXnorm,"II"), 3), p=colXn, G=k)    
+    
     df <- df_model + (k-1) +colXb*k+colXp*k+ colXn*k + k*(sum(m)-colXm)
     if (df >= n) warning(paste0(df," parameters with only ",n," observations."),call. =FALSE)
     
@@ -127,23 +134,25 @@ cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,
     ECM     <- sum(hard.z*log(z.const))
     IC$ICL <- IC$BIC+ECM
 ## GLM parameters 
-    GLModel <- list()
+    GLModel <- NULL
     if(!is.null(data)){
+      GLModel <- list()
       #coef <- SEreg(l$lmodelY)    
       for (i in seq_len(k)) {
         parameters  = list(model=l$lmodelY[[i]])
-        if (familyYname=="gaussian" |familyYname=="t") parameters <- c(parameters, list(sigma = sqrt(l$VarY[1,i])))
-        if (familyYname=="t") parameters <- c(parameters, list(t_df = t_df[i]))
+        if (familyYname=="gaussian" |familyYname=="student.t") parameters <- c(parameters, list(sigma = sqrt(l$VarY[1,i])))
+        if (familyYname=="student.t") parameters <- c(parameters, list(t_df = t_df[i]))
         if (familyYname=="Gamma") parameters <- c(parameters, list(nuY =l$nuY[i]))
         GLModel[[i]] <- parameters
       }
       names(GLModel) <- paste0("comp.",1:k)
     }
 ## Concomitant
-    concomitant <- list()
+    concomitant <- NULL
     if (!is.null(Xnorm)){
+      concomitant <- list()
        normal <- list(
-        dnorm    = norm$PX,
+        d    = norm$PX,
         mu       = norm$mu,
         Sigma    = norm$Sigma,
         model    = modelXnorm
@@ -152,21 +161,21 @@ cwm2 <- function(formulaY, data, Y, Xnorm, Xmult, Xpois, Xbin,colXm,colXn,colXb,
     }
     if (!is.null(Xmult)){
       multinomial <- list(
-        dmulti    = multi$PX,
-        prob     = multi$alpha #prob
+        d         = multi$PX,
+        probs     = multi$alpha 
       )
       concomitant <- c(concomitant, multinomial= multinomial)
     }
     if (!is.null(Xpois)){
         poisson <- list(
-        dpois    = pois$PX,
+        d         = pois$PX,
         lambda    = pois$lambda
       )
       concomitant <- c(concomitant, poisson= poisson)
     }
     if (!is.null(Xbin)){
       binomial <- list(
-        dbin    = bin$PX,
+        d    = bin$PX,
         p    = bin$p
       )
       concomitant <- c(concomitant, binomial= binomial)
